﻿using System;
using LibUsbDotNet;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace NehebkauWindowsHost
{
    public partial class frmMain : Form
    {
        private static frmMain onlyInstance;
        public static frmMain OnlyInstance
        {
            get
            {
                return onlyInstance;
            }
        }

        public frmMain()
        {
            onlyInstance = this;
            InitializeComponent();

            //AttachOnMouseMove(this);
        }

        public void AttachOnMouseMove(Control c)
        {
            c.MouseMove += new MouseEventHandler(all_MouseMove);
            foreach (Control child in c.Controls)
            {
                AttachOnMouseMove(child);
            }
        }

        public void CenterMouse()
        {
            Program.allowMouseMove = false;
            Program.MouseLastX = this.Location.X + this.Size.Width / 2;
            Program.MouseLastY = this.Location.Y + this.Size.Height / 2;
            Cursor.Position = new Point(this.Location.X + this.Size.Width / 2, this.Location.Y + this.Size.Height / 2);
            Program.allowMouseMove = true;
        }

        void all_MouseMove(object sender, MouseEventArgs e)
        {
            CenterMouse();
        }

        private void tmrUiPoll_Tick(object sender, EventArgs e)
        {
            if (NehebkauDevice.IsOpen)
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                this.WindowState = FormWindowState.Normal;
                this.TopMost = true;
                NehebkauDevice.TryReadTransfer();
            }
            else
            {
                this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
                this.TopMost = false;
                //try { NehebkauDevice.TryOpenDevice(); }
                //catch { NehebkauDevice.CloseDevice(); }
            }
        }

        private int actCnt = 0;
        delegate void AddToActivityCallback();
        public void AddToActivity()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new AddToActivityCallback(AddToActivity), new object[] { });
            }
            else
            {
                actCnt++;
                //lblActivityCnt.Text = actCnt.ToString();
                progbarActivity.ProgressBar.Value = actCnt % progbarActivity.ProgressBar.Maximum;
            }
        }

        delegate void SetStatusesCallback(bool connected, bool[] leds);
        public void SetStatuses(bool connected, bool[] leds)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new SetStatusesCallback(SetStatuses), new object[] { connected, leds });
            }
            else
            {
                lblConnected.Text = connected ? " Connected " : " Disconnected ";
                lblCapLock.Text = leds[1] ? " CAP " : " ";
                lblNumLock.Text = leds[0] ? " NUM " : " ";
                lblScrollLock.Text = leds[2] ? " SCROLL " : " ";
            }
        }

        delegate void SetTextCallback(string msg);
        public static void LogMsg(string msg)
        {
            onlyInstance.Log(msg);
        }
        public void Log(string msg)
        {
            if (txtLog.InvokeRequired)
            {
                this.Invoke(new SetTextCallback(Log), new object[] { msg });
            }
            else
            {
                txtLog.Text = Environment.TickCount.ToString() + ": " + msg + Environment.NewLine + txtLog.Text;
                txtLog.Invalidate();
                if (txtLog.Text.Length > (txtLog.MaxLength / 2)) txtLog.Text = txtLog.Text.Substring(0, txtLog.MaxLength / 4);
            }
        }

        private void tmrMouseCenter_Tick(object sender, EventArgs e)
        {
            if (NehebkauDevice.IsOpen)
            {
                CenterMouse();
            }
        }

        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            NehebkauDevice.CloseDevice();
        }
    }
}
